package org.nakedobjects.applib.util;

import junit.framework.TestCase;
import junit.framework.TestSuite;


public class TitleBufferTests extends TestCase {
    public static void main(final String[] args) {
        junit.textui.TestRunner.run(new TestSuite(TitleBufferTests.class));
    }

    String companyName;
    String name;
    String longString;
    
    TitleTestObject objectWithEmptyTitle;
    TitleTestObject objectWithNoTitle;
    TitleTestObject objectWithTitle;

    TitleBuffer t1;
    TitleBuffer t2;
    TitleBuffer t3;

    public TitleBufferTests(final String name) {
        super(name);
    }

    protected void setUp() {
        name = "Fred";
        t1 = new TitleBuffer(name);
        t2 = new TitleBuffer();
        companyName = "ABC Co.";
        objectWithTitle = new TitleTestObject();
        objectWithTitle.setupTitle(companyName);
        objectWithNoTitle = new TitleTestObject();
        objectWithEmptyTitle = new TitleTestObject();
        objectWithEmptyTitle.setupTitle("");
        longString = "This is a long title";
        t3 = new TitleBuffer(longString);
    }

    public void testAppend() {
        t1.append("");
        assertEquals("add empty string", name, t1.toString());
        t1.append("Smith");
        name += (" " + "Smith");
        assertEquals("append simple string", name, t1.toString());
        t1.append(",", "");
        assertEquals("append empty string with delimiter", name, t1.toString());
        t1.append(",", null);
        assertEquals("append null with delimiter", name, t1.toString());
        t1.append(",", "Xyz Ltd.");
        name += (", " + "Xyz Ltd.");
        assertEquals("append string with delimiter", name, t1.toString());
    }

    public void testAppendEmptyStringLeavesBufferUnchanged() {
        t1.append("");
        assertEquals(name, t1.toString());
    }

    public void testAppendEmptyStringWithJoinerLeavesBufferUnchanged() {
        t1.append(",", "");
        assertEquals(name, t1.toString());
    }

    public void testAppendInt() {
        TitleBuffer t = new TitleBuffer();
        t.append(123);
        assertEquals("123", t.toString());
    }

    public void testAppendNull() {
        t1.append(null);
        assertEquals(name, t1.toString());
    }

    public void testAppendNullWithJoinerLeavesBufferUnchanged() {
        t1.append(",", (Object) null);
        assertEquals(name, t1.toString());
    }

    public void testAppendObjectsWithJoinerAddsTitleWithJoinerAndSpace() {
        t1.append(",", objectWithTitle);
        assertEquals(name + ", " + companyName, t1.toString());
    }

    public void testAppendObjectsWithJoinerOnlyAddsTitleWhenBufferEmpty() {
        t2.append(",", objectWithTitle);
        assertEquals(name, t1.toString());
    }

    public void testAppendObjectWhereDefaultNotNeeded() {
        t1.append(objectWithTitle, "none");
        assertEquals(name + " " + companyName, t1.toString());
    }

    public void testAppendObjectWhereDefaultUsedAsObjectHasEmptyTitle() {
        t1.append(objectWithEmptyTitle, "none");
        assertEquals(name + " " + "none", t1.toString());
    }

    public void testAppendObjectWhereDefaultUsedAsReferenceIsNull() {
        t1.append((Object) null, "none");
        assertEquals(name + " " + "none", t1.toString());
    }

    public void testAppendObjectWithEmptyTitleLeavesBufferUnchanged() {
        t1.append(objectWithEmptyTitle);
        assertEquals(name, t1.toString());
    }

    public void testAppendObjectWithNoTitleLeavesBufferUnchanged() {
        t1.append(objectWithNoTitle);
        assertEquals(name, t1.toString());
    }

    public void testAppendObjectWithTitleAddTitleWithSpace() {
        t1.append(objectWithTitle);
        assertEquals(name + " " + companyName, t1.toString());
    }

    public void testAppendStringAddStringWithSpace() {
        t1.append("Smith");
        assertEquals("Fred Smith", t1.toString());
    }

    public void testAppendStringToEmptyBufferAddsStringWithoutSpace() {
        t2.append("Smith");
        assertEquals("Smith", t2.toString());
    }

    public void testAppendStringWithJoinerAddsStringWithJoinerAndSpace() {
        t1.append(",", "Smith");
        assertEquals("Fred, Smith", t1.toString());
    }

    public void testAppendToBuffer() {
        TitleBuffer t = new TitleBuffer("123");
        t.append("test");
        assertEquals("123 test", t.toString());
    }

    public void testAppendToEmpty() {
        TitleBuffer t = new TitleBuffer();
        t.append("test");
        assertEquals("test", t.toString());
    }

    public void testAppendValue() {
        TitleTestObject s = new TitleTestObject();

        t1.append(s);
        assertEquals("append empty TextString", name, t1.toString());

        //
        t1.append(new TitleTestObject("square"));
        assertEquals("append empty TextString", name + " " + "square", t1.toString());
    }

    public void testConcatEmptyStringLeavesBufferUnchanged() {
        t1.concat("");
        assertEquals(name, t1.toString());
    }

    public void testConcatObjects() {
        t1.concat(objectWithTitle);
        assertEquals(name + companyName, t1.toString());
    }

    public void testConcatObjectsWhereDefaultNotNeededAddsTitle() {
        t1.concat(objectWithTitle, "none");
        assertEquals(name + companyName, t1.toString());
    }

    public void testConcatObjectsWhereNoTitleAddDefaultTitle() {
        t1.concat(objectWithNoTitle, "none");
        assertEquals(name + "none", t1.toString());
    }

    public void testConcatObjectWhereNoTitleLeavesBufferUnchanged() {
        t1.concat(objectWithNoTitle);
        assertEquals(name, t1.toString());
    }

    public void testConcatObjectWhereTitleIsAdded() {
        t1.concat(objectWithTitle);
        assertEquals("FredABC Co.", t1.toString());
    }

    public void testConcatStringAddsString() {
        t1.concat("Smith");
        assertEquals("FredSmith", t1.toString());
    }

    public void testConstructorsWithObjectWhereDefaultIsNotNeeded() {
        t1 = new TitleBuffer(objectWithTitle, "test");
        assertEquals("ABC Co.", t1.toString());
    }

    public void testConstructorsWithObjectWhereDefaultIsUsedAsTitle() {
        t1 = new TitleBuffer(objectWithNoTitle, "test");
        assertEquals("test", t1.toString());
    }

    public void testConstructorWithObject() {
        TitleBuffer t = new TitleBuffer(objectWithTitle);
        assertEquals("ABC Co.", t.toString());
    }

    public void testConstructorWithObjectWithNoTitle() {
        TitleBuffer t = new TitleBuffer(objectWithEmptyTitle);
        assertEquals("", t.toString());
    }

    public void testConstructorWithString() {
        TitleBuffer t = new TitleBuffer("Test");
        assertEquals("Test", t.toString());
    }

    public void testDefaultConstructor() {
        TitleBuffer t = new TitleBuffer();
        assertEquals("", t.toString());
    }

    public void testTruncateHasNoEffectUntilTitleLongEnough() {
        t3.truncate(5);
        assertEquals("This is a long title", t3.toString());
    }

    public void testTruncateLimitsTitleLength() {
        t3.truncate(3);
        assertEquals("This is a...", t3.toString());
    }

    public void testTruncateMustBeAUsableLength() {
        try {
            t3.truncate(0);
            fail("Exception expected");
        } catch (IllegalArgumentException ee) {}
    }

    public void testObscureLeadingIgnoresZeroCharacters() throws Exception {
        t1.obscureLeadingCharacters('*', 0);
        assertEquals("Fred", t1.toString());
    }

    public void testObscureTrailingIgnoresZeroCharacters() throws Exception {
        t1.obscureTrailingCharacters('*', 0);
        assertEquals("Fred", t1.toString());
    }

    public void testObscureLeadingCharacters() throws Exception {
        t1.obscureLeadingCharacters('*', 2);
        assertEquals("**ed", t1.toString());
        
        t3.obscureLeadingCharacters('*', 8);
        assertEquals("********a long title", t3.toString());
    }

    public void testObscureTrailingCharacters() throws Exception {
        t1.obscureTrailingCharacters('*', 2);
        assertEquals("Fr**", t1.toString());
        
        t3.obscureTrailingCharacters('*', 8);
        assertEquals("This is a lo********", t3.toString());
    }

    public void testObscureLeadingCharactersWhereTitleShorterThanSpecified() throws Exception {
        t1.obscureLeadingCharacters('*', 6);
        assertEquals("****", t1.toString());
    }

    public void testObscureTrailingCharactersWhereTitleShorterThanSpecified() throws Exception {
        t1.obscureTrailingCharacters('*', 6);
        assertEquals("****", t1.toString());
    }

    public void testObscureAll() throws Exception {
        t1.obscureAllCharacters('*');
        assertEquals("****", t1.toString());
        
        t3.obscureAllCharacters('*');
        assertEquals("********************", t3.toString());
    }
    
    public void testFirstLineOfOnlyOneLine() throws Exception {
        t3.firstLine();
        assertEquals(longString, t3.toString());
    }

    public void testFirstLineOfTwo() throws Exception {
        TitleBuffer t = new TitleBuffer("Line one\nLine two");
        t.firstLine();
        assertEquals("Line one", t.toString());
    }

    public void testFirstLineIgnoresNewLines() throws Exception {
        TitleBuffer t = new TitleBuffer("\n\n\nLine two\nLine three");
        t.firstLine();
        assertEquals("Line two", t.toString());
    }
}
// Copyright (c) Naked Objects Group Ltd.
