package org.nakedobjects.applib.value;

import java.math.BigDecimal;

import junit.framework.TestCase;


public class MoneyTest extends TestCase {

    protected void setUp() throws Exception {
        super.setUp();
    }

    public void testAdd() {
        Money m1 = new Money(110, "pds");
        Money m2 = new Money(220, "pds");
        Money m3 = m1.add(m2);
        assertEquals(330.0, m3.doubleValue(), 0.0);
    }

    public void testAddWithCents() {
        Money m1 = new Money(110.10, "pds");
        Money m2 = new Money(220.50, "pds");
        Money m3 = m1.add(m2);
        assertEquals(330.60, m3.doubleValue(), 0.0);
    }

    public void testAddThrowsExceptionForDifferentCurrencies() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(200, "uds");
        try {
            m1.add(m2);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    public void testDoubleConstructorExpectsCurrencyToBeSpecified() {
        try {
            new Money(100.50, null);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    public void testLongConstructorExpectsCurrencyToBeSpecified() {
        try {
            new Money(100L, null);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    public void testCreateFromDouble() {
        Money m1 = new Money(100.50, "pds");
        assertEquals("100.50", m1.getAmount().toString());
        assertEquals(100.50, m1.doubleValue(), 0.0);
    }

    public void testDouble() {
        Money m1 = new Money(100, "pds");
        assertEquals(100.0, m1.doubleValue(), 0.0);
    }

    public void testDoubleConstructor() {
        Money m1 = new Money(100.15, "pds");
        assertEquals(100.15, m1.doubleValue(), 0.0);
    }

    public void testDoubleRoundingDown() {
        Money m1 = new Money(100.154, "pds");
        assertEquals(100.15, m1.doubleValue(), 0.0);
    }

    public void testDoubleRoundingUp() {
        Money m1 = new Money(100.156, "pds");
        assertEquals(100.16, m1.doubleValue(), 0.0);
    }

    public void testEqualsObject() {
        Money m1 = new Money(100.25, "pds");
        Money m2 = new Money(100.25, "pds");
        assertTrue(m1.equals(m2));
    }

    public void testEqualsObjectFailsWithDifferentAmounts() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(101, "pds");
        assertFalse(m1.equals(m2));
    }

    public void testEqualsObjectFailsWithDifferentCurrencies() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(100, "usd");
        assertFalse(m1.equals(m2));
    }

    public void testGetAmount() {
        Money m1 = new Money(100, "pds");
        assertEquals(BigDecimal.valueOf(10000, 2), m1.getAmount());
    }

    public void testHasSameCurrency() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(200, "pds");
        Money m3 = new Money(200, "usd");
        assertTrue(m1.hasSameCurrency(m2));
        assertFalse(m1.hasSameCurrency(m3));
    }

    public void testIsEqualTo() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(100, "pds");
        assertTrue(m1.isEqualTo(m2));
    }

    public void testIsEqualToFailsWithDifferentAmount() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(101, "pds");
        assertFalse(m1.isEqualTo(m2));
    }

    public void testIsEqualToThrowsExceptionWithDifferentCurrencies() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(100, "usd");
        try {
            m1.isEqualTo(m2);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    public void testIsGreaterThanZero() {
        Money m1 = new Money(1, "usd");
        assertTrue(m1.isGreaterThanZero());
    }

    public void testIsGreaterThanZeroFailsWhenZero() {
        Money m1 = new Money(0, "usd");
        assertFalse(m1.isGreaterThanZero());
    }

    public void testIsLessThan() {
        Money m1 = new Money(98, "pds");
        Money m2 = new Money(100, "pds");
        assertTrue(m1.isLessThan(m2));
    }

    public void testIsLessThanFails() {
        Money m1 = new Money(98, "pds");
        Money m2 = new Money(100, "pds");
        assertFalse(m2.isLessThan(m1));
    }

    public void testIsLessThanZero() {
        Money m1 = new Money(-1, "usd");
        assertTrue(m1.isLessThanZero());
    }

    public void testIsLessThanZeroFailsWhenZero() {
        Money m1 = new Money(0, "usd");
        assertFalse(m1.isLessThanZero());
    }

    public void testIsLessThrowsExceptionWithDifferentCurrencies() {
        Money m1 = new Money(98, "pds");
        Money m2 = new Money(100, "usd");
        try {
            m2.isLessThan(m1);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

    public void testIsZero() {
        Money m1 = new Money(0, "usd");
        assertTrue(m1.isZero());
    }

    public void testIsZeroFailsWhenPositive() {
        Money m1 = new Money(1, "usd");
        assertFalse(m1.isZero());
    }

    public void testSubtract() {
        Money m1 = new Money(300, "pds");
        Money m2 = new Money(100, "pds");
        Money m3 = m1.subtract(m2);
        assertEquals(200.0, m3.doubleValue(), 0.0);
    }

    public void testSubtractThrowsExceptionForDifferentCurrencies() {
        Money m1 = new Money(100, "pds");
        Money m2 = new Money(200, "uds");
        try {
            m1.subtract(m2);
            fail();
        } catch (IllegalArgumentException expected) {}
    }

}
// Copyright (c) Naked Objects Group Ltd.
